@@ -1,20 +1,32 @@
 import argparse
+import shutil
 import sys
 import textwrap
+import shlex
+
+import configparser
 import pkg_resources
 import requests
 
+import os.path
+
+from appdirs import user_config_dir
+
 from colorama import Fore, Style
 
 from toncli.modules.deploy_contract import ContractDeployer
 from toncli.modules.projects import ProjectBootstrapper
 from toncli.modules.utils.func.func import Func
 from toncli.modules.utils.system.argparse_fix import argv_fix
+from toncli.modules.utils.system.conf import config_file
 from toncli.modules.utils.system.log import logger
 from toncli.modules.utils.fift.cli_lib import process_build_cli_lib_command
 from toncli.modules.utils.fift.fift import Fift
 from toncli.modules.utils.lite_client.lite_client import LiteClient
 from toncli.modules.utils.transaction import run_transaction
+from toncli.modules.utils.check_hash import check_2_libs_actual, get_libs_paths
+from toncli.modules.abstract.deployer import AbstractDeployer
+from toncli.modules.deploy_wallet_contract import DeployWalletContract
 
 gr = Fore.GREEN
 bl = Fore.CYAN
@@ -27,6 +39,7 @@ def main():
 
     :return:
     '''
+
     fift_help = f'''positional arguments:
       {bl}command{rs}              Which mode to run, can be [interactive, run, sendboc]
       {gr}   interactive - default, run interactive fift
@@ -82,6 +95,13 @@ Command list, e.g. usage: toncli start wallet
 
 {bl}tointeger - parse string to integer to pass to contract in func
 
+{bl}sendboc - send file with boc info
+{gr}   "sendboc <path-to-file.boc>" - sends BOC file
+{gr}   "sendboc <path-to=file.fif> <other-params>" -  run fift file and run sendfile in lite-client (just like command "fift sendboc ...")
+
+{bl}wallet - print addresses of 2 wallets - bounceable wallet and deploy wallet
+{gr}   You can use this command only when wallet is built with commands "toncli build" or "toncli deploy"
+
 All commands can be found in https://github.com/disintar/toncli/blob/master/docs/commands.md
 
 {rs}
@@ -94,7 +114,7 @@ Credits: {gr}disintar.io{rs} team
 
     # This is concept of nft https://disintar.io
     # Nft information parse will be added in next versions of CLI
-    print("disintar.io NFT owners today say: 🙈 🙉 🙊")
+    # print("disintar.io NFT owners today say: 🙈 🙉 🙊")
 
     # TODO: add logging verbose
     parser = argparse.ArgumentParser(
@@ -103,14 +123,28 @@ Credits: {gr}disintar.io{rs} team
     parser.add_argument("-v", "--version", help="package version", action='store_true')
     subparser = parser.add_subparsers()
 
-    version_local = pkg_resources.get_distribution("toncli").version
-    try:
-        version_global = requests.get('https://pypi.org/pypi/toncli/json').json()['info']['version']
+    # version_local = pkg_resources.get_distribution("toncli").version
+    # try:
+    #     version_global = requests.get('https://pypi.org/pypi/toncli/json').json()['info']['version']
+    #
+    #     if version_global and version_global != version_local:
+    #         logger.info(update_text)
+    # except:
+    #     pass
 
-        if version_global and version_global != version_local:
-            print(update_text)
-    except:
-        pass
+    config = configparser.ConfigParser()
+    config.read(config_file)
+    config_default = config['DEFAULT']
+
+    warn = config_default.get('LIBS_WARNING') != 'False'
+
+    if warn:
+        if not check_2_libs_actual():
+            local_lib_path, global_lib_path = get_libs_paths()
+            logger.warning(
+                f"""\nIts seems that your local fift and func libs ({local_lib_path}) differs from their actual versions ({global_lib_path}). 
+You can update them automatically using "toncli update_libs" or disable this warning by changing "LIBS_WARNING" to "False" param in cofig\n\n"""
+            )
     #
     # START
     #
@@ -127,26 +161,49 @@ Credits: {gr}disintar.io{rs} team
     #
 
     parser_deploy = subparser.add_parser('deploy', description='Deploy project to blockchain')
-    parser_deploy.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet'],
+    parser_deploy.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet', 'ownnet'],
                                help='Network to deploy')
     parser_deploy.add_argument("--workchain", "-wc", default=0, type=int, help='Workchain deploy to')
     parser_deploy.add_argument("--ton", "-t", default=0.05, type=float,
                                help='How much TON will be sent to new contract')
     parser_deploy.add_argument("--update", action='store_true', help='Update cached configs of net')
+    parser_deploy.add_argument('--data-params', help='Data which you want to pass to data of your smart-contract',
+                               default="", type=str)
 
     #
     # get
     #
 
     parser_get = subparser.add_parser('get', description='Deploy project to blockchain')
-    parser_get.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet'],
+    parser_get.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet', 'ownnet'],
                             help='Network to deploy')
     parser_get.add_argument("--update", action='store_true', help='Update cached configs of net')
     parser_get.add_argument("--contracts", "-c", type=str,
                             help='Set contract name from project.yaml to run getmethod on')
+    parser_get.add_argument("--address", "-a", type=str,
+                            help='Set contract address to run get method on')
     parser_get.add_argument("--fift", "-f", type=str,
                             help='Run fift script on get output. Get output will be loaded to stack')
 
+    #
+    # send
+    #
+
+    parser_send = subparser.add_parser('send', description='Send internal transaction from deploy wallet to smc')
+    parser_send.add_argument("--amount", "-a", type=float, default=0, help='How much TON need to send')
+    parser_send.add_argument("--contracts", "-c", type=str, help='Set contract name from project.yaml to send to')
+    parser_send.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet', 'ownnet'],
+                             help='Network to use')
+    parser_send.add_argument("--update", action='store_true', help='Update cached configs of net')
+    parser_send.add_argument("--address", type=str,
+                             help='Set contract address to run get method on')
+
+    parser_send.add_argument("--mode", type=int, help='Sets transfer mode (0..255) for SENDRAWMSG')
+    parser_send.add_argument("--body", "-b", type=str,
+                             help='Path to fift file to get body from (need to set cell in the end of stack)')
+    parser_send.add_argument("--no-bounce", "-nb", type=int, help='Clears bounce flag')
+    parser_send.add_argument("--force-bounce", "-fb", type=int, help='Forces bounce flag')
+
     #
     # tointeger
     #
@@ -163,7 +220,8 @@ Credits: {gr}disintar.io{rs} team
     parser_run_transaction.add_argument("logical_time", type=str)
     parser_run_transaction.add_argument("transaction_hash", type=str)
     parser_run_transaction.add_argument("smc_address", type=str)
-    parser_run_transaction.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet'],
+    parser_run_transaction.add_argument("--net", "-n", default='testnet', type=str,
+                                        choices=['testnet', 'mainnet', 'ownnet'],
                                         help='Network to run transaction')
     parser_run_transaction.add_argument("--function", "-f", default=-1, type=int,
                                         help='Function selector on runvm (-1 - external message, 0 - internal, ...')
@@ -197,7 +255,7 @@ Credits: {gr}disintar.io{rs} team
     parser_fift = subparser.add_parser('fift', help=fift_help,
                                        formatter_class=argparse.RawDescriptionHelpFormatter,
                                        description=textwrap.dedent(fift_help))
-    parser_fift.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet'],
+    parser_fift.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet', 'ownnet'],
                              help='Network to deploy')
     parser_fift.add_argument("--workchain", "-wc", default=0, type=int, help='Workchain deploy to')
     parser_fift.add_argument("--update", action='store_true', default=False, help='Update cached configs of net')
@@ -220,7 +278,8 @@ Credits: {gr}disintar.io{rs} team
     parser_lite_client = subparser.add_parser('lite-client', help=lite_client_help,
                                               formatter_class=argparse.RawDescriptionHelpFormatter,
                                               description=textwrap.dedent(lite_client_help))
-    parser_lite_client.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet'],
+    parser_lite_client.add_argument("--net", "-n", default='testnet', type=str,
+                                    choices=['testnet', 'mainnet', 'ownnet'],
                                     help='Network to deploy')
     parser_lite_client.add_argument("--update", action='store_true', default=False, help='Update cached configs of net')
     parser_lite_client.add_argument("--lite-client-args", "-la", type=str,
@@ -230,6 +289,24 @@ Credits: {gr}disintar.io{rs} team
                                     default='',
                                     help='Pass args to lite-client command at the end')
 
+    #
+    #  SEND BOC
+    #
+    parser_sendboc = subparser.add_parser('sendboc')
+    parser_sendboc.add_argument('file', type=argparse.FileType('r'))
+    parser_sendboc.add_argument("--net", "-n", default='testnet', type=str, choices=['testnet', 'mainnet', 'ownnet'],
+                                help='Network to deploy')
+
+    #
+    #  WALLET
+    #
+    parser_wallet = subparser.add_parser('wallet')
+
+    #
+    #  UPDATE LIBS
+    #
+    parser_update_libs = subparser.add_parser('update_libs')
+
     #
     #  FUNC
     #
@@ -277,6 +354,9 @@ Credits: {gr}disintar.io{rs} team
     elif command in ['lite-client', 'lc']:
         _, kwargs = argv_fix(sys.argv, string_kwargs)
         args = parser.parse_args(['lite-client', *kwargs])
+    elif command == 'sendboc':
+        args, kwargs = argv_fix(sys.argv, string_kwargs)
+        args = parser.parse_args(['sendboc', args[2], *kwargs])
     elif command in ['func', 'fc', 'build']:
         _, kwargs = argv_fix(sys.argv, string_kwargs)
         args = parser.parse_args(['func', *kwargs])
@@ -286,6 +366,19 @@ Credits: {gr}disintar.io{rs} team
     elif command == 'get':
         _, kwargs = argv_fix(sys.argv, string_kwargs)
         args = parser.parse_args(['get', *kwargs])
+    elif command == 'addrs':
+        if 'project.yaml' not in os.listdir(os.getcwd()):
+            logger.error(f"🚫 {gr}{os.getcwd()}{rs} is not project root, there is no file {bl}project.yaml{rs} file")
+            sys.exit(0)
+
+        contract = ContractDeployer(network='ownnet')
+        addrs = contract.get_address()
+
+        for name, addr in zip(contract.project_config.contracts, addrs):
+            logger.info(f"[{gr}{name.name}{rs}] 🦄 Raw address: {bl}{addr[0]}{rs}")
+            logger.info(f"[{gr}{name.name}{rs}] 🦝 Bounceable address: {bl}{addr[1]}{rs}")
+            logger.info(f"[{gr}{name.name}{rs}] 🐏 Non-bounceable address: {bl}{addr[2]}{rs}")
+        sys.exit()
     elif command == 'tointeger':
         args, _ = argv_fix(sys.argv, string_kwargs)
         string_to_encode = " ".join(args[2:])
@@ -310,22 +403,49 @@ Credits: {gr}disintar.io{rs} team
         bootstrapper.deploy()
 
     elif command == 'deploy':
-        deployer = ContractDeployer(network=args.net, update_config=args.update, workchain=args.workchain, ton=args.ton)
+        deployer = ContractDeployer(network=args.net, update_config=args.update, workchain=args.workchain, ton=args.ton,
+                                    data_params=shlex.split(args.data_params))
         real_args, _ = argv_fix(sys.argv, string_kwargs)
-
         deployer.publish(real_args[2:])
 
     elif command == 'get':
-        deployer = ContractDeployer(network=args.net, update_config=args.update)
-        real_args, kwargs = argv_fix(sys.argv, string_kwargs)
-        deployer.get(real_args[2:], args)
 
+        real_args, kwargs = argv_fix(sys.argv, string_kwargs)
+
+        if args.address:
+            class My:
+                name = 'my-cool-smc'
+
+            deployer = AbstractDeployer()
+            deployer.network = args.net
+            deployer.update_config = args.update
+
+            deployer.get(real_args[2:], args, fake_addreses=[[My()], [[None, args.address]]])
+        else:
+            deployer = ContractDeployer(network=args.net, update_config=args.update)
+            deployer.get(real_args[2:], args)
+    elif command == 'send':
+        real_args, kwargs = argv_fix(sys.argv, string_kwargs)
+
+        if args.address:
+            class My:
+                name = 'my-cool-smc'
+
+            deployer = AbstractDeployer()
+            deployer.network = args.net
+            deployer.update_config = args.update
+            deployer.deploy_contract = DeployWalletContract(args.net, 0)
+
+            deployer.send([], args, fake_addreses=[[My()], [[None, args.address]]])
+        else:
+            deployer = ContractDeployer(network=args.net, update_config=args.update)
+            deployer.send([], args)
     elif command == 'run_transaction':
         run_transaction(args.net, args.smc_address, args.logical_time, args.transaction_hash, args.function, args.save)
         sys.exit()
 
     elif command == '-v':
-        print(f'v{version_local}')
+        logger.info(f'v{version_local}')
 
     elif command in ['fift', 'f', 'run']:
         # get real args
@@ -376,6 +496,53 @@ Credits: {gr}disintar.io{rs} team
         # If use run command instead of f run - need to change start arg parse position
         func = Func(command, kwargs=kwargs, args=args_to_pass)
         func.run()
+    elif command == 'sendboc':
+        real_args, _ = argv_fix(sys.argv, string_kwargs)
+
+        # Parse kwargs by argparse
+        kwargs = dict(args._get_kwargs())
+
+        file = kwargs['file']
+        ext = file.name.split('.')[-1]
+        if ext == 'boc':
+            lc = LiteClient(command=f"sendfile", args=[file.name],
+                            kwargs={'net': kwargs['net'], 'update': False, 'lite_client_args': '-v 1'})
+            lc.run()
+        elif ext == 'fif':
+            fift_params = real_args[2:]
+            fift = Fift(command='sendboc', args=fift_params)
+            fift.run()
+        else:
+            logger.error('This file extension is not supported. Supported extensions are .boc and .fif')
+    elif command == 'wallet':
+        contract_addr_file_name = 'build/contract_address'
+        if os.path.isfile(contract_addr_file_name):
+            bounceable_addr = ""
+            with open(contract_addr_file_name, 'r') as file:
+                addresses = file.read().split()
+                bounceable_addr = addresses[2]
+            logger.info(f"Your bounceable address is: {gr}{bounceable_addr}{rs}")
+
+            deploy_wallet_addr_dir = user_config_dir('toncli')
+            deploy_bouncable = ""
+            with open(f"{deploy_wallet_addr_dir}/wallet/build/contract_address", 'r') as file:
+                addresses = file.read().split()
+                deploy_bouncable = addresses[2]
+            logger.info(f"Your deploy wallet address is: {gr}{deploy_bouncable}{rs}")
+        else:
+            logger.error(
+                "Can't find file with address information.\n"
+                "Its seems that you haven't built your wallet yet.\n"
+                "You can do it with commands:\n"
+                "'toncli build' - to build locally\n"
+                "'toncli deploy' to build and immediately deploy it to net")
+    elif command == 'update_libs':
+        global_lib_path, local_lib_path = get_libs_paths()
+
+        shutil.copytree(f"{global_lib_path}/fift-libs", f"{local_lib_path}/fift-libs", dirs_exist_ok=True)
+        shutil.copytree(f"{global_lib_path}/func-libs", f"{local_lib_path}/func-libs", dirs_exist_ok=True)
+        logger.info(f"Succesfully copied fift and func libs\nfrom {global_lib_path}\nto {local_lib_path}")
+
     else:
         logger.error("🔎 Can't find such command")
         sys.exit()
